#!/usr/bin/python
# -*- coding: utf-8 -*-

##   Previews Reader
##   Copyright (C) 2004-2008 Mads Villadsen <maxximum@krakoa.dk>
    
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2 of the License, or
##   (at your option) any later version.
   
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
   
##   You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
##   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gobject, libxml2, os, re, string, cgi, pango, datetime
import previews_item

from globals import *
import util

try:
    import pygtk
    pygtk.require ("2.0")
    import gtk, gtk.glade, gnome, gnome.ui, gconf
except:
    print "pyGTK 2.0 not found."


reportlab_found = True
try:
    from reportlab.lib import colors
    from reportlab.platypus import *
    from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
except:
    reportlab_found = False
    print "python-reportlab not found"


COLUMN_ORDER = 0
COLUMN_TITLE = 1
COLUMN_ID_HIDDEN = 2

class pr_mainwindow:
    def __init__(self):
        self.wTree = None
        try:
            self.wTree = gtk.glade.XML (gladefile, "main_window")
        except:
            self.wTree = gtk.glade.XML (devel_gladefile, "main_window")
        self.main_window = self.wTree.get_widget ("main_window")
        try:
            self.main_window.set_icon_from_file(window_icon_file)
        except:
            pass
        self.previews_item_list = []
        self.show_star_system_items = False
        self.show_oa_items = False
        self.group_by_publisher = False
        self.show_only_new_series = False

        try:
            gtk.window_set_default_icon_from_file(window_icon_file)
        except:
            pass

        self.conf_client = gconf.client_get_default()
        self.pref_conversion_factor = self.conf_client.get_float("/apps/previews-reader/currency/conversion_factor")
        self.pref_pdf_title_line = ""
        temp = self.conf_client.get_string("/apps/previews-reader/pdf_title_line")
        if (temp != None):
            self.pref_pdf_title_line = temp

        dic = {
            "on_main_window_destroy" : gtk.main_quit,
            "on_quit_activate" : gtk.main_quit,
            "on_print_order_list1_activate" : self.print_order_list,
            "on_view_order_list_activate" : self.view_order_list,
            "on_group_by_publisher_activate" : self.group_by_publisher_activate,
            "on_show_star_system_items_activate" : self.show_star_system_items_activate,
            "on_show_oa_items_activate" : self.show_oa_items_activate,
            "on_show_only_new_series_activate" : self.show_only_new_series_activate,
            "on_shipped_items_activate": self.show_shipped_items,
            "on_about_activate" : self.about_activate,
            "on_preferences_activate" : self.preferences_activate,
            "on_open_activate" : self.open_activate,
            "on_open_treeview_row_activated" : self.open_treeview_row_activated,
            "on_treeview_row_activated" : self.show_description
            }
        self.wTree.signal_autoconnect (dic)
        
        # Favorites DOM
        try:
            self.fav_doc = libxml2.parseFile (fav_fname)
            self.fav_root = self.fav_doc.xpathEval("/favorites")[0]
            if self.fav_doc.name != fav_fname:
                print "doc.name error"
                sys.exit(1)
        except:
            self.fav_doc = libxml2.newDoc("1.0")
            self.fav_root = self.fav_doc.newChild(None, "favorites", None)

        # TODO: make a window showing favorites, allowing the user to remove from the list as well
        # dump favorites
        #faves = []
        #for n in self.fav_doc.children:
        #    if (n.name == "favorites"):
        #        for m in n.children:
        #            if (m.properties != None):
        #                for p in m.properties:
        #                    if (p.name == "title"):
        #                        #print p.content
        #                        faves.append(p.content)
        #faves.sort()
        #for fave in faves:
        #    print fave

        # setup treeview
        self.previewsTreeView = self.wTree.get_widget ("treeview1")
        self.previewsTreeModel = gtk.TreeStore (gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.previewsTreeView.set_model (self.previewsTreeModel)
        self.previewsTreeView.set_headers_visible (True)
        self.previewsTreeView.set_search_column(COLUMN_TITLE)

        renderer = gtk.CellRendererToggle()
        renderer.connect('toggled', self.order_toggled, self.previewsTreeModel)
        column = gtk.TreeViewColumn('Order', renderer, active=COLUMN_ORDER)
        column.set_resizable (True)
        self.previewsTreeView.append_column (column)
        
        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("Title", renderer, markup=COLUMN_TITLE)
        column.set_resizable (True)
        self.previewsTreeView.append_column (column)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("previews item id (hidden)", renderer, text=COLUMN_ID_HIDDEN)
        column.set_resizable (True)

        self.clipboard = gtk.clipboard_get(gtk.gdk.SELECTION_CLIPBOARD)
        self.copy_to_clipboard_text = ""

        self.previewsTreeView.show ()
        return

    def insert_row (self, model, parent, title, id):
        iter = model.insert_before (parent, None)
        item = self.previews_item_list[int(id)]

        if (self.default_orders and self.item_is_favorite(id)):
            val = item.get_title()
            val = self.clean(val)
            # does it already exist in the order DOM?
            order_node = self.order_doc.xpathEval ("/orders//order[@title='" + val + "' and @itemcode='" + item.get_itemcode() + "']")
            if (len(order_node) == 0):
                # add to order DOM
                elem = self.order_root.newChild(None, "order", None)
                elem.setProp("title", val)
                elem.setProp("itemcode", item.get_itemcode())

        order = self.item_is_order(id)

        model.set_value (iter, COLUMN_ORDER, order)
        title_to_set = cgi.escape(util.pretty_title(title))
        if (item.get_new_series()):
            title_to_set = "<i>" + title_to_set + "</i>"
        model.set_value (iter, COLUMN_TITLE, title_to_set)
        model.set_value (iter, COLUMN_ID_HIDDEN, id)
        return iter

    def fill_rows (self, previews_items):
        self.previewsTreeModel.clear()

        parent_iters = {}

        total_items_shown = 0

        star_system_match_re = re.compile("\([A-Z][A-Z][A-Z][A-Z]?[0-9]?[0-9][0-9][0-9][0-9][0-9]\)")
        oa_match_re = re.compile("\(O\/A\)")
        for i,v in enumerate(previews_items):
            if (self.show_star_system_items or star_system_match_re.search(v.get_title()) == None):
                if (self.show_oa_items or oa_match_re.search(v.get_title()) == None):
                    if (not parent_iters.has_key(v.get_publisher())):
                        tmp_iter = None
                        if (self.group_by_publisher):
                            tmp_iter = self.previewsTreeModel.insert_before (None, None)
                            self.previewsTreeModel.set_value (tmp_iter, COLUMN_ORDER, False)
                            self.previewsTreeModel.set_value (tmp_iter, COLUMN_TITLE, cgi.escape(util.pretty_title(v.get_publisher())))
                            self.previewsTreeModel.set_value (tmp_iter, COLUMN_ID_HIDDEN, -1)

                        parent_iters[v.get_publisher()] = tmp_iter

                    if (self.show_only_new_series):
                        if (v.get_new_series()):
                            iter = self.insert_row (self.previewsTreeModel,
                                                    parent_iters[v.get_publisher()],
                                                    v.get_title(),
                                                    i)
                            total_items_shown = total_items_shown +1
                    else:
                        iter = self.insert_row (self.previewsTreeModel,
                                                parent_iters[v.get_publisher()],
                                                v.get_title(),
                                                i)
                        total_items_shown = total_items_shown +1
        appbar = self.wTree.get_widget ("appbar1")
        appbar.pop()
        appbar.push(str(total_items_shown) + " items shown")
        return
	
    def clean(self, raw):
        raw = string.replace(raw, '&', '')
        raw = string.replace(raw, '<', '')
        raw = string.replace(raw, '>', '')
        raw = string.replace(raw, "'", '')
        raw = string.replace(raw, '"', '')
        return raw

    def favorite_toggled(self, widget, id):
        favorite = self.item_is_favorite(id)
        item = self.previews_item_list[int(id)]
        val = item.get_title()
        val = self.clean(val)

        if (int(id) >= 0):
            if (string.find(val, "(") >= 0):
                val = val[0:string.find(val, "(")]
            if (string.find(val, "#") >= 0):
                val = val[0:string.find(val, "#")]
            val = string.strip(val)
            val = string.replace(val, "VOL ", "VOLUME ")

            if (favorite == False):
                # set current item as a favorite
                favorite = True

                # does it already exist in favorites?
                fav_node = self.fav_doc.xpathEval ("/favorites//favorite[@title='" + val + "']")
                if (len(fav_node) == 0):
                    # add to favorites dom
                    elem = self.fav_root.newChild(None, "favorite", None)
                    elem.setProp("title", val)
            else:
                favorite = False
                # remove current item from favorites dom
                fav_node = self.fav_doc.xpathEval ("/favorites//favorite[@title='" + val + "']")
                # only remove if it hasn't already been removed
                if (len(fav_node) != 0):
                    fav_node[0].unlinkNode()

            # set new value
            widget.set_active(favorite)
            # save the dom to a file
            self.fav_doc.saveFile(fav_fname)

    def item_is_favorite(self, id):
        item = self.previews_item_list[int(id)]
        val = item.get_title()
        val = self.clean(val)

        if (int(id) >= 0):
            if (string.find(val, "(") >= 0):
                val = val[0:string.find(val, "(")]
            if (string.find(val, "#") >= 0):
                val = val[0:string.find(val, "#")]
            val = string.strip(val)
            val = string.replace(val, "VOL ", "VOLUME ")

            # does it already exist in favorites?
            fav_node = self.fav_doc.xpathEval ("/favorites//favorite[@title='" + val + "']")
            if (len(fav_node) == 1):
                return True
            else:
                return False

    def item_is_order(self, id):
        item = self.previews_item_list[int(id)]
        val = item.get_title()
        val = self.clean(val)

        if (int(id) >= 0):
            # does it already exist in orders DOM?
            order_node = self.order_doc.xpathEval ("/orders//order[@title='" + val + "' and @itemcode='" + item.get_itemcode() + "']")
            if (len(order_node) == 1):
                return True
            else:
                return False

    def order_toggled(self, cell, path, model):
        iter = model.get_iter(path)
        order = model.get_value(iter, COLUMN_ORDER)

        id =  self.previewsTreeModel.get_value (iter, COLUMN_ID_HIDDEN)
        item = self.previews_item_list[int(id)]
        val = item.get_title()
        val = self.clean(val)

        if (int(id) >= 0):
            if (order == False):
                # set status of current item as order
                order = True

                # does it already exist in the order DOM?
                order_node = self.order_doc.xpathEval ("/orders//order[@title='" + val + "' and @itemcode='" + item.get_itemcode() + "']")
                if (len(order_node) == 0):
                    # add to order DOM
                    elem = self.order_root.newChild(None, "order", None)
                    elem.setProp("title", val)
                    elem.setProp("itemcode", item.get_itemcode())
            else:
                order = False
                # remove current item from order DOM
                order_node = self.order_doc.xpathEval ("/orders//order[@title='" + val + "' and @itemcode='" + item.get_itemcode() + "']")
                # only remove if it hasn't already been removed
                if (len(order_node) != 0):
                    order_node[0].unlinkNode()

            # set new value
            model.set(iter, COLUMN_ORDER, order)
            # save the dom to a file
            self.order_doc.saveFile(self.orders_fname)
            self.default_orders = False

    def show_description (self, widget, path, column):
        iter = self.previewsTreeModel.get_iter (path)
        id =  self.previewsTreeModel.get_value (iter, COLUMN_ID_HIDDEN)
        item = self.previews_item_list[int(id)]
        val = item.get_title()
        
        descriptionTextView = self.wTree.get_widget ("textview1")
        buffer = descriptionTextView.get_buffer ()
        buffer.delete (buffer.get_start_iter (), buffer.get_end_iter ())
        iter = buffer.get_start_iter ()

        bold_tag = buffer.create_tag(name=None, weight=pango.WEIGHT_BOLD)
        italic_tag = buffer.create_tag(name=None, style=pango.STYLE_ITALIC)

        if (int(id) >= 0):
            buffer.insert(iter, "\n")
            anchor = buffer.create_child_anchor(iter)
            buffer.insert(iter, "\n\n")
            favorites_button = gtk.CheckButton()
            favorites_button.set_label("Favorite")
            favorites_button.set_active(self.item_is_favorite(id))
            favorites_button.connect("toggled", self.favorite_toggled, id)
            descriptionTextView.add_child_at_anchor(favorites_button, anchor)
            favorites_button.show_all()

            buffer.insert_with_tags(iter, util.pretty_title(item.get_title()) + " " + item.get_price(), bold_tag)
            if (item.get_comment() != ""):
                buffer.insert (iter, " (" + string.strip(item.get_comment()) + ")")
            buffer.insert (iter, "\n")
            buffer.insert_with_tags(iter, util.pretty_title(item.get_publisher()) + "\n", italic_tag)
            if (item.get_new_series()):
                buffer.insert_with_tags(iter, "New series.\n", italic_tag)
            buffer.insert (iter, "Located on page " + item.get_pagenumber() + ".")

            if (string.find(val, "(") >= 0):
                val = val[0:string.find(val, "(")]
            val = string.strip(val)
            val = string.replace(val, "VOL ", "VOLUME ")

            search_item_re = re.compile("[\t]*" + val, re.IGNORECASE)
            for e in self.previews_string_list:
                e = unicode(e, "cp1252")
                clean = e
                clean = string.replace(clean, ":", "")
                clean = string.replace(clean, ".", "")
                clean = string.replace(clean, ",", "")
                clean = string.replace(clean, "!", "")
                clean = string.replace(clean, "'S", "")
                clean = string.replace(clean, "'", "")

                search_item_match = search_item_re.match(clean)
                if (search_item_match != None):
                    buffer.insert (iter, "\n\n\n")
                    buffer.insert (iter, e)
                    break

        return

    def show_shipped_items(self, widget):
        self.copy_to_clipboard_text = ""
        view_shipped_dialogwTree = None
        try:
            view_shipped_dialogwTree = gtk.glade.XML(gladefile, "view_shipped_dialog")
        except:
            view_shipped_dialogwTree = gtk.glade.XML(devel_gladefile, "view_shipped_dialog")
        view_shipped_dialog = view_shipped_dialogwTree.get_widget("view_shipped_dialog")
        view_shipped_dialog.set_title("Shipped Items")
        title_label = view_shipped_dialogwTree.get_widget("title_label")
        title_label.set_label("<b>Shipped Items</b>")
        
        view_shipped_dialog.set_transient_for(self.main_window)
        try:
            view_shipped_dialog.set_icon_from_file(window_icon_file)
        except:
            pass

        dic = {
            "on_copy_to_clipboard_button_clicked": self.copy_to_clipboard_button_clicked
            }
        view_shipped_dialogwTree.signal_autoconnect (dic)

        # setup treeview
        shippedTreeModel = gtk.TreeStore (gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        shippedTreeModel.set_sort_column_id(6, gtk.SORT_ASCENDING) # this sorts on the filename of the order data files - which is the same as sorting on the year and month of the Previews from where the order was made. It should sort on shipping date instead

        shippedTreeView = view_shipped_dialogwTree.get_widget ("shipped_treeview")
        shippedTreeView.set_model (shippedTreeModel)
        shippedTreeView.set_headers_visible (True)
        shippedTreeView.set_search_column(COLUMN_TITLE)

        renderer = gtk.CellRendererToggle()
        renderer.connect('toggled', self.received_toggled, shippedTreeModel)
        column = gtk.TreeViewColumn('Received', renderer, active=0)
        column.set_resizable (True)
        shippedTreeView.append_column (column)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("Title", renderer, markup=1)
        column.set_resizable (True)
        shippedTreeView.append_column (column)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("Publisher", renderer, markup=2)
        column.set_resizable (True)
        shippedTreeView.append_column (column)

        renderer = gtk.CellRendererText ()
        renderer.set_property('xalign', 1.0)
        column = gtk.TreeViewColumn ("Price", renderer, markup=3)
        column.set_resizable (True)
        shippedTreeView.append_column (column)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("previews item id (hidden)", renderer, text=4)
        column.set_resizable (True)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("filename (hidden)", renderer, text=5)
        column.set_resizable (True)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("Shipping date", renderer, text=6)
        column.set_resizable (True)
        shippedTreeView.append_column (column)

        oldest_order = "9999-99"
        ship = []
        orders = os.listdir (basedir)
        if (len(orders) > 0):
            orders.sort()
            for fname in orders:
                if (fname[:6] == "order-"):
                    # Order DOM
                    order_doc = libxml2.parseFile (basedir + fname)
                    order_root = order_doc.xpathEval("/orders")[0]
                    if (order_doc.name != basedir + fname):
                        print "doc.name error"
                        sys.exit(1)

                    order_node = order_doc.xpathEval ("/orders//order[not(@received) or @received='False']/@itemcode")
                    if (len(order_node) != 0):
                        order_date = fname[6:13]
                        if (order_date < oldest_order):
                            oldest_order = order_date
                            # print "Found older order: " + oldest_order
        shippings = os.listdir (previews_dir)
        if (len(shippings) > 0):
            shippings.sort()
            for fname in shippings:
                if (fname[-12:] == "shipping.txt"):
                    date = fname[9:16]
                    if(date >= oldest_order):
                        # print "Loaded " + fname
                        f = open (previews_dir + fname)
                        #ship += f.readlines()
                        for line in f:
                            if(string.strip(line) != ""):
                                ship.append(line + "#" + fname[9:19]) # a somewhat nasty hack to tie the full shipping date to each shipped item - this extra information has to be stripped later when displaying the publisher
                        f.close()

        if (len(orders) > 0):
            for fname in orders:
                if (fname[:6] == "order-" and fname[6:13] >= oldest_order):
                    # Order DOM
                    order_doc = libxml2.parseFile (basedir + fname)
                    order_root = order_doc.xpathEval("/orders")[0]
                    if (order_doc.name != basedir + fname):
                        print "doc.name error"
                        sys.exit(1)

                    item_re = re.compile("^([A-Z][A-Z][A-Z][0-9][0-9][ ]?[0-9][0-9][0-9][0-9])\t([^\t]*)\t([^\n]*)")
                    publisher = ""

                    order_node = order_doc.xpathEval ("/orders//order[not(@received) or @received='False']/@itemcode")
                    if (len(order_node) != 0):
                        for i in order_node:
                            itemcode = string.replace(i.getContent(), ' ', '')
                            itemcode_re = re.compile("^" + itemcode)

                            for line in ship:
                                line = line.replace("\r", "")
                                item_match = item_re.search(line)
                                if (item_match == None and string.strip(line) != ""):
                                    publisher = string.strip(line.split("#")[0])

                                itemcode_match = itemcode_re.search(line)
                                if (itemcode_match != None):
                                    pi = None
                                    if (item_match != None and string.strip(line) != ""):
                                        pi = previews_item.previews_item()
                                        pi.set_itemcode(unicode(item_match.group(1), "cp1252"))
                                        pi.set_title(unicode(item_match.group(2), "cp1252"))
                                        pi.set_price(unicode(item_match.group(3), "cp1252"))
                                        pi.set_publisher(unicode(publisher, "cp1252"))

                                        # TODO: fix clipboard text to also be sorted by shipping date
                                        self.copy_to_clipboard_text += "(" + line.split("#")[-1] + ") " + pi.pretty_string_no_page() + "\n"
                                        #self.order_csv_text = self.order_csv_text + pi.get_item_code() + ";" + pi.get_title() + ";" + pi.get_price() + "\n"
                                        tree_iter = shippedTreeModel.insert_before (None, None)

                                        shippedTreeModel.set_value (tree_iter, 0, False)
                                        shippedTreeModel.set_value (tree_iter, 1, util.pretty_title(pi.get_title()))
                                        shippedTreeModel.set_value (tree_iter, 2, util.pretty_title(pi.get_publisher()))
                                        shippedTreeModel.set_value (tree_iter, 3, util.pretty_title(pi.get_price()))
                                        shippedTreeModel.set_value (tree_iter, 4, pi.get_itemcode())
                                        shippedTreeModel.set_value (tree_iter, 5, fname)
                                        shippedTreeModel.set_value (tree_iter, 6, line.split("#")[-1])

        result = view_shipped_dialog.run()
        view_shipped_dialog.destroy()

    def received_toggled(self, cell, path, model):
        iter = model.get_iter(path)
        received = model.get_value(iter, 0)
        new_received = not received
        itemcode = model.get_value (iter, 4)
        fname = model.get_value (iter, 5)
        #print received
        #print new_received
        #print itemcode
        #print fname
        order_doc = libxml2.parseFile(basedir + fname)
        order_root = order_doc.xpathEval("/orders")[0]
        if (order_doc.name != basedir + fname):
            print "doc.name error"
            sys.exit(1)

        # convert itemcode to the space separated format
        itemcode = itemcode[:5] + " " + itemcode[5:]
        order_node = order_doc.xpathEval ("/orders//order[@itemcode='" + itemcode + "']")
        if (len(order_node) == 1):
            order_node[0].setProp("received", str(new_received))
            model.set(iter, 0, new_received)

        # save the dom to a file
        order_doc.saveFile(basedir + fname)

    def print_order_list(self, widget):
        print "Not supported."
        return

    def view_order_list(self, widget):
        self.copy_to_clipboard_text = ""
        view_order_dialogwTree = None
        try:
            view_order_dialogwTree = gtk.glade.XML(gladefile, "view_order_dialog")
        except:
            view_order_dialogwTree = gtk.glade.XML(devel_gladefile, "view_order_dialog")
        view_order_dialog = view_order_dialogwTree.get_widget("view_order_dialog")
        
        view_order_dialog.set_transient_for(self.main_window)
        try:
            view_order_dialog.set_icon_from_file(window_icon_file)
        except:
            pass

        dic = {
            "on_copy_to_clipboard_button_clicked": self.copy_to_clipboard_button_clicked,
            "on_export_button_clicked" : self.export_button_clicked
            }
        view_order_dialogwTree.signal_autoconnect (dic)

        # setup treeview
        orderTreeView = view_order_dialogwTree.get_widget ("order_treeview")
        orderTreeModel = gtk.TreeStore (gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        orderTreeView.set_model (orderTreeModel)
        orderTreeView.set_headers_visible (True)
        orderTreeView.set_search_column(COLUMN_TITLE)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("Title", renderer, markup=0)
        column.set_resizable (True)
        orderTreeView.append_column (column)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("Publisher", renderer, markup=1)
        column.set_resizable (True)
        orderTreeView.append_column (column)

        renderer = gtk.CellRendererText ()
        renderer.set_property('xalign', 1.0)
        column = gtk.TreeViewColumn ("Price", renderer, markup=2)
        column.set_resizable (True)
        orderTreeView.append_column (column)

        renderer = gtk.CellRendererText ()
        renderer.set_property('xalign', 1.0)
        column = gtk.TreeViewColumn ("Page", renderer, markup=3)
        column.set_resizable (True)
        orderTreeView.append_column (column)

        float_price = 0.0

        for i,v in enumerate(self.previews_item_list):
            if (self.item_is_order(i)):
                self.copy_to_clipboard_text = self.copy_to_clipboard_text + v.pretty_string() + "\n"
                tree_iter = orderTreeModel.insert_before (None, None)
                orderTreeModel.set_value (tree_iter, 0, util.pretty_title(v.get_title()))
                orderTreeModel.set_value (tree_iter, 1, util.pretty_title(v.get_publisher()))
                orderTreeModel.set_value (tree_iter, 2, util.pretty_title(v.get_price()))
                orderTreeModel.set_value (tree_iter, 3, util.pretty_title(v.get_pagenumber()))

                try:
                    clean_price = v.get_price()
                    clean_price = string.replace(clean_price, "$", "")
                    clean_price = string.replace(clean_price, "SRP:", "")
                    clean_price = string.strip(clean_price)
                    float_price = float_price + float(clean_price)
                except:
                    # print "Invalid price: " + v.get_price()
                    pass

        stat = ""
        stat = "Total order price: <b>$" + str(round(float_price, 2)) + "</b>"
        if (self.pref_conversion_factor != 0.0):
            stat = stat + "\nTotal order price in local currency: <b>" + str(round(float_price * self.pref_conversion_factor, 2)) + "</b>"

        stat_label = view_order_dialogwTree.get_widget ("stat_label")
        stat_label.set_markup(stat)

        result = view_order_dialog.run()
        view_order_dialog.destroy()

    def about_activate(self, widget):
        about_dialogwTree = None
        try:
            about_dialogwTree = gtk.glade.XML(gladefile, "about")
        except:
            about_dialogwTree = gtk.glade.XML(devel_gladefile, "about")

        about_dialog = about_dialogwTree.get_widget("about")

        about_dialog.set_transient_for(self.main_window)

        about_dialog.set_property('name', program_name)
        about_dialog.set_property('version', program_version)

    def preferences_activate(self, widget):
        preferences_dialogwTree = None
        try:
            preferences_dialogwTree = gtk.glade.XML(gladefile, "preferences_dialog")
        except:
            preferences_dialogwTree = gtk.glade.XML(devel_gladefile, "preferences_dialog")
        preferences_dialog = preferences_dialogwTree.get_widget("preferences_dialog")
        
        preferences_dialog.set_transient_for(self.main_window)
        try:
            preferences_dialog.set_icon_from_file(window_icon_file)
        except:
            pass

        conversion_spinbutton = preferences_dialogwTree.get_widget("conversion_spinbutton")
        conversion_spinbutton.set_value(self.pref_conversion_factor)

        pdf_title_line_entry = preferences_dialogwTree.get_widget("pdf_title_line_entry")
        pdf_title_line_entry.set_text(self.pref_pdf_title_line)

        result = preferences_dialog.run()

        # get value from spinbutton and set it in gconf
        self.pref_conversion_factor = round(conversion_spinbutton.get_value(), 2)
        self.conf_client.set_float("/apps/previews-reader/currency/conversion_factor", self.pref_conversion_factor)

        # get value from entry and set it in gconf
        self.pref_pdf_title_line = pdf_title_line_entry.get_text()
        self.conf_client.set_string("/apps/previews-reader/pdf_title_line", self.pref_pdf_title_line)

        preferences_dialog.destroy()

    def open_activate(self, widget):
        open_dialogwTree = None
        try:
            open_dialogwTree = gtk.glade.XML(gladefile, "open_dialog")
        except:
            open_dialogwTree = gtk.glade.XML(devel_gladefile, "open_dialog")
        open_dialog = open_dialogwTree.get_widget("open_dialog")
        
        open_dialog.set_transient_for(self.main_window)
        try:
            open_dialog.set_icon_from_file(window_icon_file)
        except:
            pass

        dic = {
            "on_open_treeview_row_activated" : self.open_treeview_row_activated
            }
        open_dialogwTree.signal_autoconnect (dic)

        openTreeView = open_dialogwTree.get_widget ("open_treeview")
        openListModel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
        openTreeView.set_model(openListModel)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ("Previews Issue", renderer, text=0)
        openTreeView.append_column (column)

        previews = os.listdir (previews_dir)
        if (len(previews) > 0):
            previews.sort()
            previews.reverse()
            for fname in previews:
                if (fname[-13:] == "orderform.txt"):
                    date = fname[9:16]
                    iter = openListModel.append (None)
                    openListModel.set_value (iter, 0, "Previews " + " " + date)
                    openListModel.set_value (iter, 1, date)
        else:
            date = "0000-00"
            iter = openListModel.append (None)
            openListModel.set_value (iter, 0, "No Previews files found")
            openListModel.set_value (iter, 1, date)

        result = open_dialog.run()

        if (result == gtk.RESPONSE_OK):
            selected = openTreeView.get_selection().get_selected()
            iter = selected[1]
            if (iter != None):
                date_to_open = selected[0].get_value(iter, 1)
                if (date_to_open != "0000-00"):
                    #progress_dialogwTree = None
                    #try:
                    #    progress_dialogwTree = gtk.glade.XML(gladefile, "progress_dialog")
                    #except:
                    #    progress_dialogwTree = gtk.glade.XML(devel_gladefile, "progress_dialog")
                    #progress_dialog = progress_dialogwTree.get_widget("progress_dialog")
                    #progressbar = progress_dialogwTree.get_widget("progress_progress")
                    # FIXME: why isn't the progress shown until after all theses operations are done?
                    #self.timer = gtk.timeout_add (100, self.progress_timeout, progressbar)

                    # Order DOM
                    try:
                        self.orders_fname = basedir + "order-" + date_to_open + ".xml"
                        self.order_doc = libxml2.parseFile (self.orders_fname)
                        self.order_root = self.order_doc.xpathEval("/orders")[0]
                        self.default_orders = False
                        if self.order_doc.name != basedir + "order-" + date_to_open + ".xml":
                            print "doc.name error"
                            sys.exit(1)
                    except:
                        self.order_doc = libxml2.newDoc("1.0")
                        self.order_root = self.order_doc.newChild(None, "orders", None)
                        self.default_orders = True

                    self.previews_string_list = util.get_previews_data (previews_dir + "previews-" + date_to_open)
                    if (os.path.exists(previews_dir + "previews-" + date_to_open + "-newseries.txt")):
                        self.previews_item_list = util.get_orderform_items (previews_dir + "previews-" + date_to_open + "-orderform.txt", previews_dir + "previews-" + date_to_open + "-newseries.txt")
                    else:
                        self.previews_item_list = util.get_orderform_items (previews_dir + "previews-" + date_to_open + "-orderform.txt", previews_dir + "previews-" + date_to_open + "-newseries.htm")
                    self.fill_rows (self.previews_item_list)
                    self.main_window.set_title(program_name + " - " + date_to_open)

                    #gtk.timeout_remove(self.timer)
                    #self.timer = None
                    #progress_dialog.destroy()

        open_dialog.destroy()


    def progress_timeout(self, bar):
        #print ".",
        bar.pulse()

    def open_treeview_row_activated(self, treeview, path, column):
        treeview.get_parent().get_parent().get_parent().get_parent().response(gtk.RESPONSE_OK)

    def group_by_publisher_activate(self, widget):
        self.group_by_publisher = widget.get_active()
        self.fill_rows (self.previews_item_list)

    def show_star_system_items_activate(self, widget):
        self.show_star_system_items = widget.get_active()
        self.fill_rows (self.previews_item_list)

    def show_oa_items_activate(self, widget):
        self.show_oa_items = widget.get_active()
        self.fill_rows (self.previews_item_list)

    def show_only_new_series_activate(self, widget):
        self.show_only_new_series = widget.get_active()
        self.fill_rows (self.previews_item_list)

    def copy_to_clipboard_button_clicked(self, widget):
        #print "copy to clipboard: "
        #print self.copy_to_clipboard_text
        self.clipboard.set_text(self.copy_to_clipboard_text)

    def export_button_clicked(self, widget):
        save_dialogwTree = None
        try:
            save_dialogwTree = gtk.glade.XML(gladefile, "filechooserdialog")
        except:
            save_dialogwTree = gtk.glade.XML(devel_gladefile, "filechooserdialog")

        save_dialog = save_dialogwTree.get_widget("filechooserdialog")
        save_button = save_dialogwTree.get_widget("button3")
        save_button.set_label("gtk-save")

        save_dialog.set_transient_for(self.main_window)
        save_dialog.set_action(gtk.FILE_CHOOSER_ACTION_SAVE)

        if (reportlab_found):
            pdf_filter = gtk.FileFilter()
            pdf_filter.set_name("PDF")
            pdf_filter.add_pattern("*.pdf")
            save_dialog.add_filter(pdf_filter)

        csv_filter = gtk.FileFilter()
        csv_filter.set_name("CSV")
        csv_filter.add_pattern("*.csv")
        save_dialog.add_filter(csv_filter)

        try:
            save_dialog.set_icon_from_file(window_icon_file)
        except:
            pass

        result = save_dialog.run()
        if (result == gtk.RESPONSE_OK):
            temp = save_dialog.get_filename()

            if (temp != ""):
                export_format = save_dialog.get_filter().get_name()

                if (export_format == "PDF"):
                    self.export_order_list_to_pdf_file(temp)
                elif (export_format == "CSV"):
                    self.export_order_list_to_csv_file(temp)
                else:
                    print("Unknown export format selected: " + export_format)
                    

        save_dialog.destroy()

        
    def export_order_list_to_pdf_file(self, File):
        lst = []
        data = []
        styleSheet = getSampleStyleSheet()
        today = datetime.date.today()
        styRight = styleSheet['Normal']
        styRight.alignment = 2
        styTitle = styleSheet['Heading1']
        styTitle.alignment = 1
        lst.append(Paragraph(today.strftime("%d-%m-%Y"), styRight))
        lst.append(Paragraph(self.pref_pdf_title_line,styTitle))
        removeHash = re.compile( '#.*' )
        removeTitle = re.compile( '.*#' )
        removePrice = re.compile( 'SRP: ' )
        tmpData = ['Item#','Publisher','Title','Issue#','Page#','Price']
        data.append(tmpData)
        for i,v in enumerate(self.previews_item_list):
            if (self.item_is_order(i)):
               tmpData = []
               tmpData.append(v.get_itemcode())
               tmpData.append(util.pretty_title(v.get_publisher()))
               tmpData.append(removeHash.sub('', util.pretty_title(v.get_title())))
               if (util.pretty_title(v.get_title()).find("#") > 0):
                   tmpData.append(removeTitle.sub('', util.pretty_title(v.get_title())))
               else:
                   tmpData.append("")
               tmpData.append(v.get_pagenumber())
               tmpData.append(removePrice.sub('', v.get_price()))
               data.append(tmpData)
        t=Table(data,None,None)
        t.setStyle(TableStyle([('SIZE',(0,0), (-1,0), 14),
                               ('BOX', (0,0), (-1,-1), 0.25, colors.black),
                               ('INNERGRID', (0,0), (-1,-1), 0.25, colors.black) ]))
        lst.append(t)
        SimpleDocTemplate(File).build(lst)

    def export_order_list_to_csv_file(self, filename):
        f = open(filename, "w")

        for i,v in enumerate(self.previews_item_list):
            if (self.item_is_order(i)):
                f.write("\"")
                f.write(v.get_itemcode())
                f.write("\"")
                f.write(",")
                f.write("\"")
                f.write(util.pretty_title(v.get_publisher()))
                f.write("\"")
                f.write(",")
                f.write("\"")
                f.write(util.pretty_title(v.get_title()))
                f.write("\"")
                f.write(",")
                f.write("\"")
                f.write(v.get_pagenumber())
                f.write("\"")
                f.write(",")
                f.write("\"")
                f.write(v.get_price())
                f.write("\"")
                f.write("\n")

        f.close()

