#!/usr/bin/python

##   Previews Reader
##   Copyright (C) 2004-2008 Mads Villadsen <maxximum@krakoa.dk>
    
##   This program is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2 of the License, or
##   (at your option) any later version.
   
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
   
##   You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
##   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import urllib2, os, sys, re, string, datetime

month_mapping = ['', 'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December']
short_month_mapping = ['', 'JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', 'SEP', 'OCT', 'NOV', 'DEC']
caps_reverse_month_mapping = {'JANUARY' : '01', 'FEBRUARY' : '02', 'MARCH' : '03', 'APRIL' : '04', 'MAY' : '05', 'JUNE' : '06', 'JULY' : '07', 'AUGUST' : '08', 'SEPTEMBER' : '09', 'OCTOBER' : '10', 'NOVEMBER' : '11', 'DECEMBER' : '12'}
DEBUG = False

def debug_print(out):
    if (DEBUG):
        print(out)
    else:
        pass

def pprogress():
    sys.stdout.write("#")
    sys.stdout.flush()

def download_single_file(url, dest_fname):
    retval = True
    if (not(os.path.exists(dest_fname))):
        try:
            uf = urllib2.urlopen(url)
            string_list = []
            temp = uf.readline()
            while (temp != ""):
                string_list.append(temp)
                temp = uf.readline()
            uf.close()

            f = open(dest_fname, "w")
            f.writelines(string_list)
            f.close()
        except urllib2.HTTPError, (code):
            debug_print(code)
            retval = False
    else:
        debug_print(dest_fname + " already exists")
    return retval
    

def download_previews_files(year, month):
    alternate_worlds_archive_url = "http://www.alternateworlds.com.au/catalogue_items/"
    staffars_serier_archive_url = "http://www.staffars.se/previews/"
    failed_list = []
    string_list = []

    # Try Staffars Serier
    # convert year to previews volume number
    volume = str(int(year) - 1990)
    dest_fname = "previews-" + year + "-" + month + ".txt"
    debug_print("trying to download from staffars serier: " + dest_fname)
    pprogress()
    if (not(download_single_file(staffars_serier_archive_url + "previews" + volume + "-" + month + ".txt", dest_fname))):
        failed_list.append(dest_fname)

    dest_fname = "previews-" + year + "-" + month + "-adult.txt"
    debug_print("trying to download from staffars serier: " + dest_fname)
    pprogress()
    if (not(download_single_file(staffars_serier_archive_url + "adult" + volume + "-" + month + ".txt", dest_fname))):
        failed_list.append(dest_fname)

    dest_fname = "previews-" + year + "-" + month + "-marvel.txt"
    debug_print("trying to download from staffars serier: " + dest_fname)
    pprogress()
    if (not(download_single_file(staffars_serier_archive_url + "marvel" + volume + "-" + month + ".txt", dest_fname))):
        failed_list.append(dest_fname)

    # Try Alternate Worlds
    dest_fname = "previews-" + year + "-" + month + ".txt"
    debug_print("trying to download from alternate worlds: " + dest_fname)
    pprogress()
    if (not(download_single_file(alternate_worlds_archive_url + "previews" + month + year[2:4] + ".txt", dest_fname))):
        failed_list.append(dest_fname)

    dest_fname = "previews-" + year + "-" + month + "-adult.txt"
    debug_print("trying to download from alternate worlds: " + dest_fname)
    pprogress()
    if (not(download_single_file(alternate_worlds_archive_url + "adult" + month + year[2:4] + ".txt", dest_fname))):
        failed_list.append(dest_fname)

    # the Diamond Comics Previews order form archive uses a few different naming conventions, so we need to try all possibilities
    # FIXME: this doesn't handle 1998, 1999, and 2001
    orderform_archive_url = "http://previews.diamondcomics.com/support/previews_docs/orderforms/archive/"

    dest_fname = "previews-" + year + "-" + month + "-newseries.htm"
    dest_fname_pdf = "previews-" + year + "-" + month + "-newseries.pdf"
    debug_print("trying to download from diamond comics: " + dest_fname)
    pprogress()
    if (not(download_single_file(orderform_archive_url + year + "/" + month_mapping[int(month)] + "%20" + year + "/" + "NewSeries.htm", dest_fname))):
        pprogress()
        if (not(download_single_file(orderform_archive_url + year + "/" + short_month_mapping[int(month)] + year[2:4] + "/" + short_month_mapping[int(month)] + year[2:4] + "newseries.htm", dest_fname))):
            pprogress()
            if (not(download_single_file(orderform_archive_url + year + "/" + short_month_mapping[int(month)] + year[2:4] + "/" + short_month_mapping[int(month)] + year[2:4] + "newseries.pdf", dest_fname_pdf))):
                pprogress()
                if (not(download_single_file(orderform_archive_url + year + "/" + short_month_mapping[int(month)] + year[2:4] + "/" + short_month_mapping[int(month)] + year[2:4] + "_Newseries.pdf", dest_fname_pdf))):
                    failed_list.append(dest_fname)

    dest_fname = "previews-" + year + "-" + month + "-cancel.txt"
    debug_print("trying to download from diamond comics: " + dest_fname)
    pprogress()
    if (not(download_single_file(orderform_archive_url + year + "/" + month_mapping[int(month)] + "%20" + year + "/" + "Cancel.txt", dest_fname))):
        pprogress()
        if (not(download_single_file(orderform_archive_url + year + "/" + short_month_mapping[int(month)] + year[2:4] + "/" + short_month_mapping[int(month)] + year[2:4] + "cancel.txt", dest_fname))):
            pprogress()
            if (not(download_single_file(orderform_archive_url + year + "/" + short_month_mapping[int(month)] + year[2:4] + "/" + short_month_mapping[int(month)] + year[2:4] + "_Cancel.txt", dest_fname))):
                failed_list.append(dest_fname)

    dest_fname = "previews-" + year + "-" + month + "-orderform.txt"
    
    debug_print("trying to download consumer order form from diamond comics")
    debug_print("    trying: " + orderform_archive_url + year + "/" + short_month_mapping[int(month)] + year[2:4] + "/" + short_month_mapping[int(month)] + year[2:4] + "_COF.txt")
    pprogress()
    if (not(download_single_file(orderform_archive_url + year + "/" + short_month_mapping[int(month)] + year[2:4] + "/" + short_month_mapping[int(month)] + year[2:4] + "_COF.txt", dest_fname))):
        debug_print("    failed")
        debug_print("    trying: " + orderform_archive_url + year + "/" + short_month_mapping[int(month)] + year[2:4] + "/" + short_month_mapping[int(month)] + year[2:4] + "cof.txt")
        pprogress()
        if (not(download_single_file(orderform_archive_url + year + "/" + short_month_mapping[int(month)] + year[2:4] + "/" + short_month_mapping[int(month)] + year[2:4] + "cof.txt", dest_fname))):
            debug_print("    failed")
            debug_print("    trying: " + orderform_archive_url + year + "/" + month_mapping[int(month)] + "%20" + year + "/" + "cof.txt")
            pprogress()
            if (not(download_single_file(orderform_archive_url + year + "/" + month_mapping[int(month)] + "%20" + year + "/" + "cof.txt", dest_fname))):
                debug_print("    failed")
                debug_print("    trying: " + orderform_archive_url + month + "_" + year[2:4] + "/" + "cof.txt")
                pprogress()
                if (not(download_single_file(orderform_archive_url + month + "_" + year[2:4] + "/" + "cof.txt", dest_fname))):
                    debug_print("    failed")
                    debug_print("    trying: " + orderform_archive_url + year + "/" + month + "_" + year[2:4] + "/" + "cof.txt")
                    pprogress()
                    if (not(download_single_file(orderform_archive_url + year + "/" + month + "_" + year[2:4] + "/" + "cof.txt", dest_fname))):
                        debug_print("    failed")
                        debug_print("    trying: " + orderform_archive_url + month + "_" + year[2:4] + "/" + month + "_" + year[2:4] + "_cof.txt")
                        pprogress()
                        if (not(download_single_file(orderform_archive_url + month + "_" + year[2:4] + "/" + month + "_" + year[2:4] + "_cof.txt", dest_fname))):
                            debug_print("    failed")
                            debug_print("    trying: " + orderform_archive_url + year + "/" + month + "_" + year[2:4] + "/" + month + "_" + year[2:4] + "_cof.txt")
                            pprogress()
                            if (not(download_single_file(orderform_archive_url + year + "/" + month + "_" + year[2:4] + "/" + month + "_" + year[2:4] + "_cof.txt", dest_fname))):
                                debug_print("    failed")
                                debug_print("    trying: " + orderform_archive_url + year + "/" + month + "_" + year[2:4] + "/" + month + "_" + year[2:4] + "cof.txt")
                                pprogress()
                                if (not(download_single_file(orderform_archive_url + year + "/" + month + "_" + year[2:4] + "/" + month + "_" + year[2:4] + "cof.txt", dest_fname))):
                                    debug_print("    failed")
                                    debug_print("    trying: " + "http://www.diamondcomics.com/previews/orderform/" +  year[2:4] + "_" + month + "cof.txt")
                                    pprogress()
                                    if (not(download_single_file("http://www.diamondcomics.com/previews/orderform/" +  year[2:4] + "_" + month + "cof.txt", dest_fname))):
                                        debug_print("    failed")
                                        failed_list.append(dest_fname)


    # get the index page of the shipping list archives
    debug_print("getting shipping list archive page")
    pprogress()
    shipping_archive_url = "http://www.diamondcomics.com/shipping/archive/" + year + "/" + month
    try:
        uf = urllib2.urlopen("http://www.diamondcomics.com/public/default.asp?t=3")
        lines = uf.readlines()
        uf.close()
        
        # find the relevant url
        href_re = re.compile("a href=/shipping/archive/" + year + "/" + month + "([^\.]*)\.txt")
        for l in lines:
            href_match = href_re.findall(l)
            if (href_match != None):
                for match in href_match:
                    dest_fname = "previews-" + year + "-" + month + "-" + match[0:2] + "-shipping.txt"
                    debug_print("trying to download " + shipping_archive_url + match + ".txt")
                    pprogress()
                    if (not(download_single_file(shipping_archive_url + match + ".txt", dest_fname))):
                        failed_list.append(dest_fname)
    except:
        debug_print("error getting shipping lists")

    # get the current shipping list page
    debug_print("getting current shipping list page")
    pprogress()
    shipping_list_url = "http://www.diamondcomics.com/shipping/shipping_"
    try:
        uf = urllib2.urlopen("http://www.previewsworld.com/public/default.asp?t=1&m=1&c=6&s=428&ai=65208")
        lines = uf.readlines()
        uf.close()
        # find the relevant url
        href_re = re.compile("href=\"http://www.diamondcomics.com/shipping/shipping_([^\.]*)\.txt\"")
        for l in lines:
            href_match = href_re.findall(l)
            if (href_match != None):
                for match in href_match:
                    if (len(match) == 6):
                        if (match[0:2] == month):
                            # correct month
                            dest_fname = "previews-" + year + "-" + month + "-" + match[2:4] + "-shipping.txt"
                            debug_print("trying to download " + shipping_list_url + match + ".txt")
                            pprogress()
                            if (not(download_single_file(shipping_list_url + match + ".txt", dest_fname))):
                                failed_list.append(dest_fname)
    except:
        debug_print("error getting current shipping list")

    # see if the current order form and current NewSeries.htm are a match for the requested year and month
    #current_order_form = "http://previews.diamondcomics.com/support/previews_docs/orderforms/cof.txt"
    #current_new_series_htm = "http://previews.diamondcomics.com/support/previews_docs/orderforms/newseries.htm"
    #current_cancel = "http://previews.diamondcomics.com/support/previews_docs/orderforms/cancel.txt"
    current_order_form = "http://previews.diamondcomics.com/support/previews_docs/orderforms/" + month_mapping[int(month)][0:3].upper() + year[2:4] + "cof.txt"
    current_new_series_htm = "http://previews.diamondcomics.com/support/previews_docs/orderforms/" + month_mapping[int(month)][0:3].upper() + year[2:4] + "newseries.htm"
    current_cancel = "http://previews.diamondcomics.com/support/previews_docs/orderforms/" + month_mapping[int(month)][0:3].upper() + year[2:4] + "cancel.txt"
    pprogress()
    try:
        uf = urllib2.urlopen(current_order_form)
        foundit = False
        # look at the first 50 lines to see if we can find something matching the expected item code the requested month and year
        for i in range(50):
            l = uf.readline()
            if (l.startswith("\t" + month_mapping[int(month)][0:3].upper() + year[2:4])):
                #print "current order form is from the correct year and month"
                foundit = True
        uf.close()
        if (foundit):
            dest_fname = "previews-" + year + "-" + month + "-orderform.txt"
            debug_print("    trying: " + current_order_form)
            pprogress()
            if (not(download_single_file(current_order_form, dest_fname))):
                debug_print("    failed")
            else:
                # we got it now so remove it from the failed list
                failed_list.remove(dest_fname)

            dest_fname = "previews-" + year + "-" + month + "-newseries.htm"
            debug_print("    trying: " + current_new_series_htm)
            pprogress()
            if (not(download_single_file(current_new_series_htm, dest_fname))):
                debug_print("    failed")
            else:
                # we got it now so remove it from the failed list
                failed_list.remove(dest_fname)

            dest_fname = "previews-" + year + "-" + month + "-cancel.txt"
            debug_print("    trying: " + current_cancel)
            pprogress()
            if (not(download_single_file(current_cancel, dest_fname))):
                debug_print("    failed")
            else:
                # we got it now so remove it from the failed list
                failed_list.remove(dest_fname)
    except:
        debug_print("error reading current order form")

    # get the current shipping list and save it with a constant file name
    current_shipping = "http://www.diamondcomics.com/shipping/newreleases.txt"
    pprogress()
    try:
        uf = urllib2.urlopen(current_shipping)
        foundit = False
        s_year = ""
        s_month = ""
        s_day = ""        
        shipping_date_re = re.compile("Shipping This Week: (.*)")
        # look at the first 10 lines to see if we can find the date of this shipping list
        for i in range(10):
            l = uf.readline()
            shipping_date_match = shipping_date_re.findall(l)
            if (shipping_date_match != None):
                for match in shipping_date_match:
                    splits = match.split(" ")
                    if (len(splits) == 3):
                        s_year = splits[2].strip()
                        try:
                            s_month = caps_reverse_month_mapping[splits[0].strip().upper()]
                        except:
                            s_month = ""
                        s_date = splits[1].strip().strip(",")
                        if (len(s_date) == 1):
                            s_date = "0" + s_date
                        # sanity check data
                        if (len(s_year) == 4 and len(s_month) == 2 and len(s_date) == 2):
                            #print("looking for: " + year + " " + month)
                            #print("found      : " + year + " " + month)
                            # check that we have found the one the user is asking for
                            if (s_year == year and s_month == month):
                                foundit = True
                    else:
                        splits = match.split("/")
                        if (len(splits) == 3):
                            s_year = splits[2].strip()
                            s_date = splits[1].strip()
                            if (len(s_date) == 1):
                                s_date = "0" + s_date
                            s_month = splits[0].strip()
                            if (len(s_year) == 4 and len(s_month) == 2 and len(s_date) == 2):
                                #print("looking for: " + year + " " + month)
                                #print("found      : " + year + " " + month)
                                # check that we have found the one the user is asking for
                                if (s_year == year and s_month == month):
                                    foundit = True
                            

        uf.close()
        if (foundit):
            debug_print("found shipping list for: " + s_year + "-" + s_month + "-" + s_date)
            download_single_file(current_shipping, "previews-" + s_year + "-" + s_month + "-" + s_date + "-shipping.txt")
    except:
        debug_print("error reading current shipping list")
        
        
    print("")
    if (len(failed_list) > 0):
        for s in failed_list:
            print("Failed to download " + s)

if __name__ == '__main__':
    if (len(sys.argv) != 3):
        print("Wrong number of arguments - please specify both year and month (for example 2004 09)")
    else:
        year = sys.argv[1]
        month = sys.argv[2]
        if (len(year) != 4):
            print("Year must be of the form YYYY, for example 2004")
        else:
            if (len(month) != 2):
                print("Month must be of the form MM, for example 09")
            else:
                download_previews_files(year, month)
